@@ -4,7 +4,7 @@
  *		Routines for handling specialized SET variables.
  *
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -753,79 +753,41 @@ check_session_authorization(char **newval, void **extra, GucSource source)
 	if (*newval == NULL)
 		return true;
 
-	if (InitializingParallelWorker)
+	if (!IsTransactionState())
 	{
 		/*
-		 * In parallel worker initialization, we want to copy the leader's
-		 * state even if it no longer matches the catalogs. ParallelWorkerMain
-		 * already installed the correct role OID and superuser state.
+		 * Can't do catalog lookups, so fail.  The result of this is that
+		 * session_authorization cannot be set in postgresql.conf, which seems
+		 * like a good thing anyway, so we don't work hard to avoid it.
 		 */
-		roleid = GetSessionUserId();
-		is_superuser = GetSessionUserIsSuperuser();
+		return false;
 	}
-	else
-	{
-		if (!IsTransactionState())
-		{
-			/*
-			 * Can't do catalog lookups, so fail.  The result of this is that
-			 * session_authorization cannot be set in postgresql.conf, which
-			 * seems like a good thing anyway, so we don't work hard to avoid
-			 * it.
-			 */
-			return false;
-		}
 
+	/* Look up the username */
+	roleTup = SearchSysCache1(AUTHNAME, PointerGetDatum(*newval));
+	if (!HeapTupleIsValid(roleTup))
+	{
 		/*
 		 * When source == PGC_S_TEST, we don't throw a hard error for a
-		 * nonexistent user name or insufficient privileges, only a NOTICE.
-		 * See comments in guc.h.
+		 * nonexistent user name, only a NOTICE.  See comments in guc.h.
 		 */
-
-		/* Look up the username */
-		roleTup = SearchSysCache1(AUTHNAME, PointerGetDatum(*newval));
-		if (!HeapTupleIsValid(roleTup))
+		if (source == PGC_S_TEST)
 		{
-			if (source == PGC_S_TEST)
-			{
-				ereport(NOTICE,
-						(errcode(ERRCODE_UNDEFINED_OBJECT),
-						 errmsg("role \"%s\" does not exist", *newval)));
-				return true;
-			}
-			GUC_check_errmsg("role \"%s\" does not exist", *newval);
-			return false;
-		}
-
-		roleform = (Form_pg_authid) GETSTRUCT(roleTup);
-		roleid = roleform->oid;
-		is_superuser = roleform->rolsuper;
-
-		ReleaseSysCache(roleTup);
-
-		/*
-		 * Only superusers may SET SESSION AUTHORIZATION a role other than
-		 * itself. Note that in case of multiple SETs in a single session, the
-		 * original authenticated user's superuserness is what matters.
-		 */
-		if (roleid != GetAuthenticatedUserId() &&
-			!GetAuthenticatedUserIsSuperuser())
-		{
-			if (source == PGC_S_TEST)
-			{
-				ereport(NOTICE,
-						(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
-						 errmsg("permission will be denied to set session authorization \"%s\"",
-								*newval)));
-				return true;
-			}
-			GUC_check_errcode(ERRCODE_INSUFFICIENT_PRIVILEGE);
-			GUC_check_errmsg("permission denied to set session authorization \"%s\"",
-							 *newval);
-			return false;
+			ereport(NOTICE,
+					(errcode(ERRCODE_UNDEFINED_OBJECT),
+					 errmsg("role \"%s\" does not exist", *newval)));
+			return true;
 		}
+		GUC_check_errmsg("role \"%s\" does not exist", *newval);
+		return false;
 	}
 
+	roleform = (Form_pg_authid) GETSTRUCT(roleTup);
+	roleid = roleform->oid;
+	is_superuser = roleform->rolsuper;
+
+	ReleaseSysCache(roleTup);
+
 	/* Set up "extra" struct for assign_session_authorization to use */
 	myextra = (role_auth_extra *) malloc(sizeof(role_auth_extra));
 	if (!myextra)
@@ -874,16 +836,6 @@ check_role(char **newval, void **extra, GucSource source)
 		roleid = InvalidOid;
 		is_superuser = false;
 	}
-	else if (InitializingParallelWorker)
-	{
-		/*
-		 * In parallel worker initialization, we want to copy the leader's
-		 * state even if it no longer matches the catalogs. ParallelWorkerMain
-		 * already installed the correct role OID and superuser state.
-		 */
-		roleid = GetCurrentRoleId();
-		is_superuser = session_auth_is_superuser;
-	}
 	else
 	{
 		if (!IsTransactionState())
@@ -923,8 +875,13 @@ check_role(char **newval, void **extra, GucSource source)
 
 		ReleaseSysCache(roleTup);
 
-		/* Verify that session user is allowed to become this role */
-		if (!is_member_of_role(GetSessionUserId(), roleid))
+		/*
+		 * Verify that session user is allowed to become this role, but skip
+		 * this in parallel mode, where we must blindly recreate the parallel
+		 * leader's state.
+		 */
+		if (!InitializingParallelWorker &&
+			!is_member_of_role(GetSessionUserId(), roleid))
 		{
 			if (source == PGC_S_TEST)
 			{
