@@ -3,7 +3,7 @@
  * nodeMergejoin.c
  *	  routines supporting merge joins
  *
- * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -893,10 +893,11 @@ ExecMergeJoin(PlanState *pstate)
 
 						if (compareResult == 0)
 							node->mj_JoinState = EXEC_MJ_JOINTUPLES;
-						else if (compareResult < 0)
+						else
+						{
+							Assert(compareResult < 0);
 							node->mj_JoinState = EXEC_MJ_NEXTOUTER;
-						else	/* compareResult > 0 should not happen */
-							elog(ERROR, "mergejoin input data is out of order");
+						}
 						break;
 					case MJEVAL_NONMATCHABLE:
 
@@ -1086,7 +1087,7 @@ ExecMergeJoin(PlanState *pstate)
 
 					node->mj_JoinState = EXEC_MJ_JOINTUPLES;
 				}
-				else if (compareResult > 0)
+				else
 				{
 					/* ----------------
 					 *	if the new outer tuple didn't match the marked inner
@@ -1105,6 +1106,7 @@ ExecMergeJoin(PlanState *pstate)
 					 *	no more inners, no more matches are possible.
 					 * ----------------
 					 */
+					Assert(compareResult > 0);
 					innerTupleSlot = node->mj_InnerTupleSlot;
 
 					/* reload comparison data for current inner */
@@ -1138,8 +1140,6 @@ ExecMergeJoin(PlanState *pstate)
 							return NULL;
 					}
 				}
-				else			/* compareResult < 0 should not happen */
-					elog(ERROR, "mergejoin input data is out of order");
 				break;
 
 				/*----------------------------------------------------------
