@@ -3,7 +3,7 @@
  * tsvector.c
  *	  I/O functions for tsvector
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
@@ -22,7 +22,7 @@
 
 typedef struct
 {
-	WordEntry	entry;			/* must be first, see compareentry */
+	WordEntry	entry;			/* must be first! */
 	WordEntryPos *pos;
 	int			poslen;			/* number of elements in pos */
 } WordEntryIN;
@@ -78,19 +78,16 @@ uniquePos(WordEntryPos *a, int l)
 	return res + 1 - a;
 }
 
-/*
- * Compare two WordEntry structs for qsort_arg.  This can also be used on
- * WordEntryIN structs, since those have WordEntry as their first field.
- */
+/* Compare two WordEntryIN values for qsort */
 static int
 compareentry(const void *va, const void *vb, void *arg)
 {
-	const WordEntry *a = (const WordEntry *) va;
-	const WordEntry *b = (const WordEntry *) vb;
+	const WordEntryIN *a = (const WordEntryIN *) va;
+	const WordEntryIN *b = (const WordEntryIN *) vb;
 	char	   *BufferStr = (char *) arg;
 
-	return tsCompareString(&BufferStr[a->pos], a->len,
-						   &BufferStr[b->pos], b->len,
+	return tsCompareString(&BufferStr[a->entry.pos], a->entry.len,
+						   &BufferStr[b->entry.pos], b->entry.len,
 						   false);
 }
 
@@ -170,6 +167,12 @@ uniqueentry(WordEntryIN *a, int l, char *buf, int *outbuflen)
 	return res + 1 - a;
 }
 
+static int
+WordEntryCMP(WordEntry *a, WordEntry *b, char *buf)
+{
+	return compareentry(a, b, buf);
+}
+
 
 Datum
 tsvectorin(PG_FUNCTION_ARGS)
@@ -488,7 +491,7 @@ tsvectorrecv(PG_FUNCTION_ARGS)
 		 * But make sure the buffer is large enough first.
 		 */
 		while (hdrlen + SHORTALIGN(datalen + lex_len) +
-			   sizeof(uint16) + npos * sizeof(WordEntryPos) >= len)
+			   (npos + 1) * sizeof(WordEntryPos) >= len)
 		{
 			len *= 2;
 			vec = (TSVector) repalloc(vec, len);
@@ -502,7 +505,7 @@ tsvectorrecv(PG_FUNCTION_ARGS)
 
 		datalen += lex_len;
 
-		if (i > 0 && compareentry(&vec->entries[i],
+		if (i > 0 && WordEntryCMP(&vec->entries[i],
 								  &vec->entries[i - 1],
 								  STRPTR(vec)) <= 0)
 			needSort = true;
@@ -534,7 +537,7 @@ tsvectorrecv(PG_FUNCTION_ARGS)
 					elog(ERROR, "position information is misordered");
 			}
 
-			datalen += sizeof(uint16) + npos * sizeof(WordEntryPos);
+			datalen += (npos + 1) * sizeof(WordEntry);
 		}
 	}
 
