@@ -3,7 +3,7 @@
  * be-gssapi-common.h
  *       Definitions for GSSAPI authentication and encryption handling
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/libpq/be-gssapi-common.h
@@ -20,7 +20,7 @@
 #include <gssapi/gssapi.h>
 #endif
 
-void		pg_GSS_error(int severity, const char *errmsg,
+extern void pg_GSS_error(const char *errmsg,
 						 OM_uint32 maj_stat, OM_uint32 min_stat);
 
 #endif							/* BE_GSSAPI_COMMON_H */
