@@ -3,7 +3,7 @@
  * walsender_private.h
  *	  Private definitions from replication/walsender.c.
  *
- * Portions Copyright (c) 2010-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2010-2021, PostgreSQL Global Development Group
  *
  * src/include/replication/walsender_private.h
  *
@@ -98,30 +98,15 @@ typedef struct
 	XLogRecPtr	lsn[NUM_SYNC_REP_WAIT_MODE];
 
 	/*
-	 * Status of data related to the synchronous standbys.  Waiting backends
-	 * can't reload the config file safely, so checkpointer updates this value
-	 * as needed. Protected by SyncRepLock.
+	 * Are any sync standbys defined?  Waiting backends can't reload the
+	 * config file safely, so checkpointer updates this value as needed.
+	 * Protected by SyncRepLock.
 	 */
-	bits8		sync_standbys_status;
+	bool		sync_standbys_defined;
 
 	WalSnd		walsnds[FLEXIBLE_ARRAY_MEMBER];
 } WalSndCtlData;
 
-/* Flags for WalSndCtlData->sync_standbys_status */
-
-/*
- * Is the synchronous standby data initialized from the GUC?  This is set the
- * first time synchronous_standby_names is processed by the checkpointer.
- */
-#define SYNC_STANDBY_INIT			(1 << 0)
-
-/*
- * Is the synchronous standby data defined?  This is set when
- * synchronous_standby_names has some data, after being processed by the
- * checkpointer.
- */
-#define SYNC_STANDBY_DEFINED		(1 << 1)
-
 extern WalSndCtlData *WalSndCtl;
 
 
@@ -136,7 +121,6 @@ extern int	replication_yylex(void);
 extern void replication_yyerror(const char *str) pg_attribute_noreturn();
 extern void replication_scanner_init(const char *query_string);
 extern void replication_scanner_finish(void);
-extern bool replication_scanner_is_replication_command(void);
 
 extern Node *replication_parse_result;
 
