@@ -1016,11 +1016,11 @@ select pg_get_viewdef('shoe'::regclass,0) as prettier;
 -- check multi-row VALUES in rules
 --
 
-create table rules_src(f1 int, f2 int default 0);
-create table rules_log(f1 int, f2 int, tag text, id serial);
+create table rules_src(f1 int, f2 int);
+create table rules_log(f1 int, f2 int, tag text);
 insert into rules_src values(1,2), (11,12);
 create rule r1 as on update to rules_src do also
-  insert into rules_log values(old.*, 'old', default), (new.*, 'new', default);
+  insert into rules_log values(old.*, 'old'), (new.*, 'new');
 update rules_src set f2 = f2 + 1;
 update rules_src set f2 = f2 * 10;
 select * from rules_src;
@@ -1028,48 +1028,28 @@ select * from rules_log;
 create rule r2 as on update to rules_src do also
   values(old.*, 'old'), (new.*, 'new');
 update rules_src set f2 = f2 / 10;
-create rule r3 as on insert to rules_src do also
-  insert into rules_log values(null, null, '-', default), (new.*, 'new', default);
-insert into rules_src values(22,23), (33,default);
 select * from rules_src;
 select * from rules_log;
-create rule r4 as on delete to rules_src do notify rules_src_deletion;
+create rule r3 as on delete to rules_src do notify rules_src_deletion;
+\d+ rules_src
 
 --
 -- Ensure an aliased target relation for insert is correctly deparsed.
 --
-create rule r5 as on insert to rules_src do instead insert into rules_log AS trgt SELECT NEW.* RETURNING trgt.f1, trgt.f2;
-create rule r6 as on update to rules_src do instead UPDATE rules_log AS trgt SET tag = 'updated' WHERE trgt.f1 = new.f1;
-
---
--- Check deparse disambiguation of INSERT/UPDATE/DELETE targets.
---
-create rule r7 as on delete to rules_src do instead
-  with wins as (insert into int4_tbl as trgt values (0) returning *),
-       wupd as (update int4_tbl trgt set f1 = f1+1 returning *),
-       wdel as (delete from int4_tbl trgt where f1 = 0 returning *)
-  insert into rules_log AS trgt select old.* from wins, wupd, wdel
-  returning trgt.f1, trgt.f2;
-
--- check display of all rules added above
+create rule r4 as on insert to rules_src do instead insert into rules_log AS trgt SELECT NEW.* RETURNING trgt.f1, trgt.f2;
+create rule r5 as on update to rules_src do instead UPDATE rules_log AS trgt SET tag = 'updated' WHERE trgt.f1 = new.f1;
 \d+ rules_src
 
 --
--- Test implicit LATERAL references to old/new in rules
+-- Also check multiassignment deparsing.
 --
-CREATE TABLE rule_t1(a int, b text DEFAULT 'xxx', c int);
-CREATE VIEW rule_v1 AS SELECT * FROM rule_t1;
-CREATE RULE v1_ins AS ON INSERT TO rule_v1
-  DO ALSO INSERT INTO rule_t1
-  SELECT * FROM (SELECT a + 10 FROM rule_t1 WHERE a = NEW.a) tt;
-CREATE RULE v1_upd AS ON UPDATE TO rule_v1
-  DO ALSO UPDATE rule_t1 t
-  SET c = tt.a * 10
-  FROM (SELECT a FROM rule_t1 WHERE a = OLD.a) tt WHERE t.a = tt.a;
-INSERT INTO rule_v1 VALUES (1, 'a'), (2, 'b');
-UPDATE rule_v1 SET b = upper(b);
-SELECT * FROM rule_t1;
-DROP TABLE rule_t1 CASCADE;
+create table rule_t1(f1 int, f2 int);
+create table rule_dest(f1 int, f2 int[], tag text);
+create rule rr as on update to rule_t1 do instead UPDATE rule_dest trgt
+  SET (f2[1], f1, tag) = (SELECT new.f2, new.f1, 'updated'::varchar)
+  WHERE trgt.f1 = new.f1 RETURNING new.*;
+\d+ rule_t1
+drop table rule_t1, rule_dest;
 
 --
 -- check alter rename rule
